% % Example in 
%% Scialanga, Ampountolas - 
%% Interpolating Constrained Control of Interconnected Systems
%% CTS 2018
%
% from
% Narendra, K.S., Oleng, N., and Mukhopadhyay, S. (2006).
% Decentralised adaptive control with partial communication.
% IEE Proceedings – Control Theory and Applications
% 153(5), 546–555.

% System matrices:              A \in R^{12x12},
%                               B \in R^{12x6},
% State constraints:            -6 <= x <= 6;
% Control constraints:          -0.4 <= u <= 0.4.

warning off
clear all
close all
clc

% number subsystems
no_subs = 6;
% number of realisations
r = 1;
% state components
n = zeros(1, no_subs);
% control components
m = zeros(1, no_subs);
for i = 1:no_subs
    n(i) = 2;
    m(i) = 1;
end

fprintf('\n - COMPUTING DECENTRALISED CONTROL FOR A SYSTEM DECOMPOSED IN %d SUBSYSTEMS WITH INTERCONNECTIONS - \n',no_subs);

% interconnections
a = 0.005; % 0.5
b = 0.004; % 0.4

% State and control matrices
a13  = a; a15  = a; a17  = a; a19  = a; a1_11 = a;
a31  = b; a35  = b; a37  = b; a39  = b; a311 = b;
a51  = a; a53  = a; a57  = a; a59  = a; a511 = a;
a71  = b; a73  = b; a75  = b; a79  = b; a711 = b;
a91  = a; a93  = a; a95  = a; a97  = a; a911 = a;
a11_1 = b; a113 = b; a115 = b; a117 = b; a119 = b;

Ac{1} = [0.2   1.0   a13   0.0   a15   0.0   a17   0.0   a19   0.0   a1_11  0.0;  % row1
      0.2   0.5   0.0   0.0   0.0   0.0   0.0   0.0   0.0   0.0   0.0   0.0;  % row2
      a31   0.0  -0.5  -0.5   a35   0.0   a37   0.0   a39   0.0   a311  0.0;  % row3
      0.0   0.0   0.5  -0.25  0.0   0.0   0.0   0.0   0.0   0.0   0.0   0.0;  % row4
      a51   0.0   a53   0.0   0.2   1.0   a57   0.0   a59   0.0   a511  0.0;  % row5
      0.0   0.0   0.0   0.0   0.2   0.5   0.0   0.0   0.0   0.0   0.0   0.0;  % row6
      a71   0.0   a73   0.0   a75   0.0  -0.5  -0.5   a79   0.0   a711  0.0;  % row7
      0.0   0.0   0.0   0.0   0.0   0.0   0.5  -0.25  0.0   0.0   0.0   0.0;  % row8
      a91   0.0   a93   0.0   a95   0.0   a97   0.0   0.2   1.0   a911  0.0;  % row9
      0.0   0.0   0.0   0.0   0.0   0.0   0.0   0.0   0.2   0.5   0.0   0.0;  % row10
      a11_1  0.0   a113  0.0   a115  0.0   a117  0.0   a119  0.0  -0.5  -0.5;  % row11
      0.0   0.0   0.0   0.0   0.0   0.0   0.0   0.0   0.0   0.0   0.5  -0.25; % row12
      ];

Bc{1} = [1.0   0.0   0.0   0.0   0.0   0.0;
      0.0   0.0   0.0   0.0   0.0   0.0;
      0.0   2.0   0.0   0.0   0.0   0.0;
      0.0   0.0   0.0   0.0   0.0   0.0;
      0.0   0.0   1.0   0.0   0.0   0.0;
      0.0   0.0   0.0   0.0   0.0   0.0;
      0.0   0.0   0.0   2.0   0.0   0.0;
      0.0   0.0   0.0   0.0   0.0   0.0;
      0.0   0.0   0.0   0.0   1.0   0.0;
      0.0   0.0   0.0   0.0   0.0   0.0;
      0.0   0.0   0.0   0.0   0.0   2.0;
      0.0   0.0   0.0   0.0   0.0   0.0;];

nx = size(Ac{1},1); % state dimension
nu = size(Bc{1},2); % input dimension  

% LOCAL state and control constraints
Xmax = 6 * ones(no_subs,1); Umax = 0.4 * ones(no_subs,1);
Fx = cell(no_subs,1);
Fu = cell(no_subs,1);
gx = cell(no_subs,1);
gu = cell(no_subs,1);
X = cell(no_subs,1);
U = cell(no_subs,1);


for i = 1 : no_subs
    Fx{i} = [eye(n(i)); - eye(n(i))];
    Fu{i} = [eye(m(i)); - eye(m(i))];
    gx{i} = Xmax(i) * ones(2*n(i),1);
    gu{i} = Umax(i) * ones(2*m(i),1);
    X{i} = [Fx{i} gx{i}];
    U{i} = [Fu{i} gu{i}];
end

% objective matrices
Qc = eye(nx);      
Rc = 0.00001 * eye(nu);        


A = cell(no_subs,r);
B = cell(no_subs,r);
D = cell(no_subs,r);
[A, B, D] = cen2dec(Ac,Bc, n, m); 


N = 20; % time steps

x0 = [-2.1408 -5.5940 -2.5030 5.9000 3.2160 5.3800 5.3202 -3.4933 -6.0000 6.0000 -6.0000 -6.0000]';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   %   %   %   %   %   %   %   %   %   %   %   %   %   %   %   %   %   %   %   %   %   %   %   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% centralised IC
fprintf('\n\n>>>centralised IC<<<\n\n');


% State constraints
Fxc = blkdiag(Fx{:});
gxc = cat(1, gx{:});
Xc = [Fxc gxc];

 
% %Input constraints
Fuc = blkdiag(Fu{:});
guc = cat(1, gu{:});
Uc = [Fuc guc];

% LQR
KC = [0.2249    1.0623    0.0050    0.0000    0.0050    0.0000    0.0050    0.0000    0.0050    0.0000    0.0050    0.0000
      0.0020    0.0000   -0.2760   -0.2370    0.0020    0.0000    0.0020    0.0000    0.0020    0.0000    0.0020    0.0000
      0.0050    0.0000    0.0050    0.0000    0.2249    1.0623    0.0050    0.0000    0.0050    0.0000    0.0050    0.0000
      0.0020    0.0000    0.0020    0.0000    0.0020    0.0000   -0.2760   -0.2370    0.0020    0.0000    0.0020    0.0000
      0.0050    0.0000    0.0050    0.0000    0.0050    0.0000    0.0050    0.0000    0.2249    1.0623    0.0050    0.0000
      0.0020    0.0000    0.0020    0.0000    0.0020    0.0000    0.0020    0.0000    0.0020    0.0000   -0.2760   -0.2370];

%% COMPUTATION INVARIANT SETS
% Compute the maximal positively invariant set
fprintf('\n Computing the inner maximal invariant set...');
tmax = 92;
% % The three following sets are identical
%
% [OmegaC,tstar,fd] = oinfsetcl(Ac,Bc,[],X,U,[],-KC,tmax);
% F_OmegaC = Oinf(:,1:end-1);
% g_OmegaC = Oinf(:,end);
F_OmegaC = [1.0000         0         0         0         0         0         0         0         0         0         0         0
         0         0         0         0    1.0000         0         0         0         0         0         0         0
         0         0         0         0         0         0         0         0    1.0000         0         0         0
   -1.0000         0         0         0         0         0         0         0         0         0         0         0
         0         0         0         0   -1.0000         0         0         0         0         0         0         0
         0         0         0         0         0         0         0         0   -1.0000         0         0         0
   -0.2249   -1.0623   -0.0050   -0.0000   -0.0050   -0.0000   -0.0050   -0.0000   -0.0050   -0.0000   -0.0040   -0.0000
   -0.0020   -0.0000    0.2760    0.2370   -0.0020   -0.0000   -0.0020   -0.0000   -0.0020   -0.0000   -0.0020   -0.0000
   -0.0050   -0.0000   -0.0050   -0.0000   -0.2249   -1.0623   -0.0050   -0.0000   -0.0050   -0.0000   -0.0050   -0.0000
   -0.0020   -0.0000   -0.0020   -0.0000   -0.0020   -0.0000    0.2760    0.2370   -0.0020   -0.0000   -0.0020   -0.0000
   -0.0050   -0.0000   -0.0050   -0.0000   -0.0050   -0.0000   -0.0050   -0.0000   -0.2249   -1.0623   -0.0050   -0.0000
   -0.0020   -0.0000   -0.0020   -0.0000   -0.0020   -0.0000   -0.0020   -0.0000   -0.0020   -0.0000    0.2760    0.2370
    0.2249    1.0623    0.0050    0.0000    0.0050    0.0000    0.0050    0.0000    0.0050    0.0000    0.0040    0.0000
    0.0020    0.0000   -0.2760   -0.2370    0.0020    0.0000    0.0020    0.0000    0.0020    0.0000    0.0020    0.0000
    0.0050    0.0000    0.0050    0.0000    0.2249    1.0623    0.0050    0.0000    0.0050    0.0000    0.0050    0.0000
    0.0020    0.0000    0.0020    0.0000    0.0020    0.0000   -0.2760   -0.2370    0.0020    0.0000    0.0020    0.0000
    0.0050    0.0000    0.0050    0.0000    0.0050    0.0000    0.0050    0.0000    0.2249    1.0623    0.0050    0.0000
    0.0020    0.0000    0.0020    0.0000    0.0020    0.0000    0.0020    0.0000    0.0020    0.0000   -0.2760   -0.2370
    0.0000    0.0001    0.1329   -0.0664    0.0000    0.0001   -0.0001    0.0001    0.0000    0.0001   -0.0001    0.0001
    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001
    0.0000    0.0001   -0.0001    0.0001    0.0000    0.0001    0.1329   -0.0664    0.0000    0.0001   -0.0001    0.0001
    0.0001    0.0003   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001
    0.0000    0.0001   -0.0001    0.0001    0.0000    0.0001   -0.0001    0.0001    0.0000    0.0001    0.1329   -0.0664
   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001
   -0.0000   -0.0001    0.0001   -0.0001   -0.0000   -0.0001   -0.1329    0.0664   -0.0000   -0.0001    0.0001   -0.0001
   -0.0001   -0.0003    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001
    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001
    0.0001    0.0003   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001
   -0.0000   -0.0001   -0.1329    0.0664   -0.0000   -0.0001    0.0001   -0.0001   -0.0000   -0.0001    0.0001   -0.0001
   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001
   -0.0001   -0.0003    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001
   -0.2069   -0.5171   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001    0.0001    0.0003   -0.0002    0.0001
    0.0000    0.0001    0.1329   -0.0664    0.0000    0.0001   -0.0001    0.0001    0.0000    0.0001   -0.0001    0.0001
    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001
    0.0001    0.0003   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001
    0.2069    0.5171    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001   -0.0001   -0.0003    0.0002   -0.0001
   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001
    0.0001    0.0003   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001
   -0.0000   -0.0001   -0.1329    0.0664   -0.0000   -0.0001    0.0001   -0.0001   -0.0000   -0.0001    0.0001   -0.0001
   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001
   -0.2069   -0.5171   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001    0.0001    0.0003   -0.0002    0.0001
   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001
    0.0000    0.0001   -0.0001    0.0001    0.0000    0.0001    0.1329   -0.0664    0.0000    0.0001   -0.0001    0.0001
   -0.0001   -0.0003    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001
    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001
    0.2069    0.5171    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001   -0.0001   -0.0003    0.0002   -0.0001
   -0.0001   -0.0003    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001
    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001
   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001
   -0.0000   -0.0001    0.0001   -0.0001   -0.0000   -0.0001   -0.1329    0.0664   -0.0000   -0.0001    0.0001   -0.0001
   -0.2069   -0.5171   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001    0.0001    0.0003   -0.0002    0.0001
    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001
    0.0000    0.0001   -0.0001    0.0001    0.0000    0.0001   -0.0001    0.0001    0.0000    0.0001    0.1329   -0.0664
    0.2069    0.5171    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001   -0.0001   -0.0003    0.0002   -0.0001
   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001
   -0.0001   -0.0003    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001
    0.0000    0.0001    0.1329   -0.0664    0.0000    0.0001   -0.0001    0.0001    0.0000    0.0001   -0.0001    0.0001
    0.0000    0.0001   -0.0001    0.0001    0.0000    0.0001    0.1329   -0.0664    0.0000    0.0001   -0.0001    0.0001
   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001
    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001
    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001
    0.0000    0.0001   -0.0001    0.0001    0.0000    0.0001   -0.0001    0.0001    0.0000    0.0001    0.1329   -0.0664
    0.0000    0.0001   -0.0001    0.0001    0.0000    0.0001    0.1329   -0.0664    0.0000    0.0001   -0.0001    0.0001
    0.0001    0.0003   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001
   -0.0000   -0.0001   -0.1329    0.0664   -0.0000   -0.0001    0.0001   -0.0001   -0.0000   -0.0001    0.0001   -0.0001
   -0.0000   -0.0001    0.0001   -0.0001   -0.0000   -0.0001    0.0001   -0.0001   -0.0000   -0.0001   -0.1329    0.0664
   -0.2069   -0.5171   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001    0.0001    0.0003   -0.0002    0.0001
   -0.0000   -0.0001   -0.1329    0.0664   -0.0000   -0.0001    0.0001   -0.0001   -0.0000   -0.0001    0.0001   -0.0001
   -0.2069   -0.5171   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001    0.0001    0.0003   -0.0002    0.0001
    0.0001    0.0003   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001
   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001
   -0.0001   -0.0003    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001
    0.0001    0.0003   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001
    0.2069    0.5171    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001   -0.0001   -0.0003    0.0002   -0.0001
   -0.0000   -0.0001    0.0001   -0.0001   -0.0000   -0.0001   -0.1329    0.0664   -0.0000   -0.0001    0.0001   -0.0001
   -0.0001   -0.0003    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001
    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001
    0.0000    0.0001   -0.0001    0.0001    0.0000    0.0001    0.1329   -0.0664    0.0000    0.0001   -0.0001    0.0001
   -0.0000   -0.0001   -0.1329    0.0664   -0.0000   -0.0001    0.0001   -0.0001   -0.0000   -0.0001    0.0001   -0.0001
   -0.2069   -0.5171   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001    0.0001    0.0003   -0.0002    0.0001
    0.0001    0.0003   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001
   -0.0001   -0.0003    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001
    0.0000    0.0001    0.1329   -0.0664    0.0000    0.0001   -0.0001    0.0001    0.0000    0.0001   -0.0001    0.0001
    0.0000    0.0001   -0.0001    0.0001    0.0000    0.0001    0.1329   -0.0664    0.0000    0.0001   -0.0001    0.0001
    0.2069    0.5171    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001   -0.0001   -0.0003    0.0002   -0.0001
   -0.0001   -0.0003    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001
    0.0000    0.0001   -0.0001    0.0001    0.0000    0.0001   -0.0001    0.0001    0.0000    0.0001    0.1329   -0.0664
    0.2069    0.5171    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001   -0.0001   -0.0003    0.0002   -0.0001
   -0.0000   -0.0001    0.0001   -0.0001   -0.0000   -0.0001   -0.1329    0.0664   -0.0000   -0.0001    0.0001   -0.0001
   -0.0001   -0.0003    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001
    0.0000    0.0001   -0.0001    0.0001    0.0000    0.0001   -0.0001    0.0001    0.0000    0.0001    0.1329   -0.0664
    0.0001    0.0003   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001
   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001
   -0.0001   -0.0003    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001
   -0.2069   -0.5171   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001    0.0001    0.0003   -0.0002    0.0001
    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001
   -0.0000   -0.0001   -0.1329    0.0664   -0.0000   -0.0001    0.0001   -0.0001   -0.0000   -0.0001    0.0001   -0.0001
    0.0001    0.0003   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001
    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001
    0.0001    0.0003   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001   -0.2069   -0.5171   -0.0003    0.0001
   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001
   -0.0000   -0.0001    0.0001   -0.0001   -0.0000   -0.0001   -0.1329    0.0664   -0.0000   -0.0001    0.0001   -0.0001
   -0.0001   -0.0003    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001
   -0.2069   -0.5171   -0.0003    0.0001    0.0001    0.0003   -0.0003    0.0001    0.0001    0.0003   -0.0002    0.0001
   -0.0001   -0.0003    0.0003   -0.0001    0.2069    0.5171    0.0003   -0.0001   -0.0001   -0.0003    0.0003   -0.0001];
g_OmegaC = [6.0 * ones(6,1); 0.40 * ones(99,1)];
OmegaC = [F_OmegaC g_OmegaC];
% F_OmegaC = Oinf(:, 1:end-1);
% g_OmegaC = Oinf(:,end);
PC = Polyhedron(F_OmegaC(:,1:3),g_OmegaC);

display ('DONE.')


% Compute the maximal robust control invariant set
fprintf('\n Computing the outer maximal robust control invariant set...');
% tmax = 50;
% [PhiC,tstar,fd] = kinfset(Ac,Bc,[],X,U,[],X,tmax,[],[]); %max. robust control inv. set
PhiC = [-0.0050         0   -0.0050         0   -0.0050         0   -0.0050         0   -0.2000   -1.0000   -0.0050         0    6.4000
    0.0050         0    0.0050         0    0.0050         0    0.0050         0    0.2000    1.0000    0.0050         0    6.4000
   -0.0050         0   -0.0050         0   -0.2000   -1.0000   -0.0050         0   -0.0050         0   -0.0050         0    6.4000
    0.0050         0    0.0050         0    0.2000    1.0000    0.0050         0    0.0050         0    0.0050         0    6.4000
   -0.2000   -1.0000   -0.0050         0   -0.0050         0   -0.0050         0   -0.0050         0   -0.0040         0    6.4000
    0.2000    1.0000    0.0050         0    0.0050         0    0.0050         0    0.0050         0    0.0040         0    6.4000
    1.0000         0         0         0         0         0         0         0         0         0         0         0    6.0000
         0    1.0000         0         0         0         0         0         0         0         0         0         0    6.0000
         0         0    1.0000         0         0         0         0         0         0         0         0         0    6.0000
         0         0         0    1.0000         0         0         0         0         0         0         0         0    6.0000
         0         0         0         0    1.0000         0         0         0         0         0         0         0    6.0000
         0         0         0         0         0    1.0000         0         0         0         0         0         0    6.0000
         0         0         0         0         0         0    1.0000         0         0         0         0         0    6.0000
         0         0         0         0         0         0         0    1.0000         0         0         0         0    6.0000
         0         0         0         0         0         0         0         0    1.0000         0         0         0    6.0000
         0         0         0         0         0         0         0         0         0    1.0000         0         0    6.0000
         0         0         0         0         0         0         0         0         0         0    1.0000         0    6.0000
         0         0         0         0         0         0         0         0         0         0         0    1.0000    6.0000
   -1.0000         0         0         0         0         0         0         0         0         0         0         0    6.0000
         0   -1.0000         0         0         0         0         0         0         0         0         0         0    6.0000
         0         0   -1.0000         0         0         0         0         0         0         0         0         0    6.0000
         0         0         0   -1.0000         0         0         0         0         0         0         0         0    6.0000
         0         0         0         0   -1.0000         0         0         0         0         0         0         0    6.0000
         0         0         0         0         0   -1.0000         0         0         0         0         0         0    6.0000
         0         0         0         0         0         0   -1.0000         0         0         0         0         0    6.0000
         0         0         0         0         0         0         0   -1.0000         0         0         0         0    6.0000
         0         0         0         0         0         0         0         0   -1.0000         0         0         0    6.0000
         0         0         0         0         0         0         0         0         0   -1.0000         0         0    6.0000
         0         0         0         0         0         0         0         0         0         0   -1.0000         0    6.0000
         0         0         0         0         0         0         0         0         0         0         0   -1.0000    6.0000];
F_PhiC = PhiC(:,1:end-1);
g_PhiC = PhiC(:,end);
PhiC = [F_PhiC g_PhiC];
P1C = Polyhedron(F_PhiC(:,1:3),g_PhiC);
display ('DONE.')

% plot invariant sets
figure
plot(PC, 'color', 'red')
hold on
plot(P1C, 'color','yellow', 'alpha', 0.2)
plot3(x0(1), x0(2), x0(3),'*','MarkerSize',10,'LineWidth',2)
hold off
xlabel('x_1')
ylabel('x_2')
zlabel('x_3')


fprintf('\n Computing centralised interpolating control...\n');
% allocate history matrices
cX = zeros(nx,N);  % state
cU = zeros(nu,N);  % input
 
% cX    - computed states in N - steps ;
% cU    - implemented IC for N - steps ;
% cS    - computed interpolating coefficient in N - step.


[cX, cU, cS] = cIC(Ac{1}, Bc{1}, -KC, Xc, Uc, OmegaC, PhiC, [], N, x0, []);
display('DONE.')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   %   %   %   %   %   %   %   %   %   %   %   %   %   %   %   %   %   %   %   %   %   %   %   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%% decentralised IC
fprintf('\n\n>>>decentralised IC<<<\n');

M = 1; % horizon for control in outer set

W = couplingconstraints(Fx, gx);
Omega = cell(1, no_subs);
Phi = cell(1, no_subs);

Q = cell(1, no_subs);
R = cell(1, no_subs);
K = cell(1, no_subs);

for i = 1: no_subs
   fprintf('\n\n COMPUTATIONS FOR %d -TH SUBSYSTEM...\n',i');   
   
   % Weighting matrices
   Q{i} = eye(n(i));
   R{i} = 0.00001 * eye(m(i));  
   if mod(i,2) == 1
       K{i} = [-0.2249   -1.0623];
   else
       K{i} = [0.2760    0.2370];
   end
   
   %% COMPUTATION INVARIANT SETS
   % Compute the maximal positively invariant set
   fprintf('\n Computing the inner maximal invariant set...');
   [Omega{i},tstar,fd] = oinfsetcl(A{i,:},B{i,:},D{i,:},X{i},U{i},W{i},K{i},30);
   F_Omega{i} = Omega{i}(:,1:end-1);
   g_Omega{i} = Omega{i}(:,end);
   P{i} = Polyhedron(F_Omega{i},g_Omega{i});
   display ('DONE.')
   
   % Compute the maximal robust controlled invariant set
   fprintf('\n Computing the outer maximal robust control invariant set...');
   tmax = 20;
   [Phi{i},tstar,fd] = kinfset(A{i,:},B{i,:},D{i,:},X{i},U{i},W{i},X{i},tmax);%max. robust control inv. set
   F_Phi{i} = Phi{i}(:,1:end-1);
   g_Phi{i} = Phi{i}(:,end);
   P1{i} = Polyhedron (F_Phi{i}, g_Phi{i}(:,end));
   display ('DONE.')

   % plot invariant sets
   label_x = ['x' num2str(sum(n(1:i)) -1)];
   label_y = ['x' num2str(sum(n(1:i)))];
   figure
   plot(P{i}, 'color', 'red')
   hold on
   plot(P1{i}, 'color','yellow', 'alpha', 0.2)
   plot(x0(sum(n(1:i)) -1), x0(sum(n(1:i))),'*')
   hold off
   str = sprintf('Invariant sets S%d',i);
   title (str, 'FontSize', 14, 'FontWeight', 'bold')
   xlabel(label_x)
   ylabel(label_y)
end

% allocate history matrices
dX = zeros(nx,N+1); % state
dU = zeros(nu,N); % input
dS = zeros(no_subs,N);

[dX, dU, dS] = dIC(A, B, D, K, X, U, W, Omega, Phi, N, x0, []);

%% PLOTs: STATE, CONTROL AND INTERPOLATING COEFFICIENT EVOLUTION

% states and control evolution

tvec = 0:N;
plus_x = 0.2;
plus_u = 0.05;

% states
figure
for i = 1:nx
subplot(6,2,i);
hold on;
stairs(tvec,cX(i,:), 'k--');
stairs(tvec,dX(i,:), 'm-');
axis([0,N,-(6+plus_x),6+plus_x]); 
xlabel('t'); ylabel(['x_{' num2str(i) '}']);
hold off;
legend('cIC', 'dIC');
grid on;
end
 

% input
figure
for i = 1:nu
    subplot(3,2,i);
    hold on;
    stairs(1:N,cU(i,:), 'k--');
    stairs(1:N,dU(i,:), 'm-');
    axis([1,N,-(0.4+plus_u),(0.4+plus_u)]); grid on;
    set(gca,'yTick',-1:.2:1);
    xlabel('t'); ylabel(['u_{' num2str(i) '}']);
    hold off;
    legend('cIC', 'dIC');
    grid on;
end

% interpolating coefficient
 leg{1} = ['cS'];
for i = 1:no_subs
    leg{i+1} = ['s_' num2str(i)];
end

figure
stairs(cS,'b')
hold on;
stairs(dS')
xlabel('Time')
ylabel('s')
ymax = max(max(dS));
axis([1 N -.1 ymax+0.1]);
grid on;
legend(leg)

